@@ -1,67 +1,26 @@
+import os
 import sys
 import time
 from collections import namedtuple
 from dynamicmethod import dynamicmethod
+
+from pybk8500.utils import parse_number
 from pybk8500.send_cmd import CommunicationManager
 from pybk8500 import commands
 
 
-__all__ = ['parse_number', 'ProfileManager', 'ProfileRow', 'Profile']
+__all__ = ['parse_number', 'ProfileManager', 'ProfileRow', 'Profile', 'main']
 
 
 SetRemoteOperation = commands.SetRemoteOperation
 LoadSwitch = commands.LoadSwitch
 ReadInputVoltageCurrentPowerState = commands.ReadInputVoltageCurrentPowerState
+SetMode = commands.SetMode
+ReadMode = commands.ReadMode
+CommandStatus = commands.CommandStatus
 
 
-UNIT_CONVERT = {
-    'T': 10**12, 'Terra': 10**12,
-    'G': 10**9, 'Giga': 10**9,
-    'M': 10**6, 'Mega': 10**6,
-    'k': 10**3, 'kilo': 10**3,
-    'c': 10**-2, 'centi': 10**-2,
-    'm': 10**-3, 'milli': 10**-3,
-    'µ': 10**-6, 'micro': 10**-6,
-    'n': 10**-9, 'nano': 10**-9,
-    'p': 10**-12, 'pico': 10**-12,
-    }
-
-
-def parse_number(value):
-    """Convert the given value to a number"""
-    modifier = 1
-    unit = ''
-    try:
-        value, modifier, unit = value.split(' ', 2)
-    except (ValueError, TypeError, Exception):
-        try:
-            value, modifier = value.split(' ', 1)
-            modifier, unit = modifier[:-1], modifier[-1]  # Convert ms to m s
-        except (ValueError, TypeError, Exception):
-            pass
-
-    # Get the modifier
-    modifier = UNIT_CONVERT.get(modifier, modifier)
-
-    # Get the value
-    try:
-        value = int(value)
-    except (ValueError, TypeError, Exception):
-        try:
-            value = float(value)
-        except (ValueError, TypeError, Exception):
-            pass
-
-    # Convert the value to normal units
-    try:
-        value = value * modifier
-    except (ValueError, TypeError, Exception):
-        pass
-
-    return value, unit
-
-
-ProfileRow = namedtuple('ProfileRow', ['command', 'value', 'time'])
+ProfileRow = namedtuple('ProfileRow', ['command', 'value', 'timeout'])
 
 
 class Profile(list):
@@ -69,7 +28,7 @@ class Profile(list):
 
     ProfileRow = ProfileRow
     HEADER = 'Command,Value,Run Time (s)'
-    ROW_FORMAT = '{row.command},{row.value},{row.time}'
+    ROW_FORMAT = '{row.command},{row.value},{row.timeout}'
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -95,11 +54,11 @@ class Profile(list):
 
     @classmethod
     def parse_value(cls, value):
-        return parse_number(value)[0]
+        return parse_number(value)
 
     @classmethod
     def parse_timeout(cls, value):
-        timeout = parse_number(value)[0]
+        timeout = parse_number(value)
         if not timeout or isinstance(timeout, str):
             timeout = 0
         return timeout
@@ -148,7 +107,7 @@ class Profile(list):
 
 
 class ProfileManager(CommunicationManager):
-    def start_remote(self):
+    def setup_remote(self, *args, **kwargs):
         """Start the remote operation and set the load switch to off."""
         # Set to remote (Must start with this command for running remote)
         cmd = SetRemoteOperation(operation='Remote')  # or operation=1
@@ -158,7 +117,7 @@ class ProfileManager(CommunicationManager):
         cmd = LoadSwitch(operation='Off')  # or operation=0
         self.send_wait(cmd, timeout=1, print_recv=True)
 
-    def stop_remote(self):
+    def teardown_remote(self, *args, **kwargs):
         """Stop the remote operation to the front panel and set the load switch to off."""
         # Set the load Off
         cmd = LoadSwitch(operation='Off')  # or operation=0
@@ -170,11 +129,13 @@ class ProfileManager(CommunicationManager):
 
     def __enter__(self):
         super().__enter__()
-        self.start_remote()
+        if self.is_connected():
+            self.setup_remote()
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        self.stop_remote()
+        if self.is_connected():
+            self.teardown_remote()
         return super().__exit__(exc_type, exc_val, exc_tb)
 
     def __init__(self, connection=None, parser=None, com=None, baudrate=None, **kwargs):
@@ -187,6 +148,7 @@ class ProfileManager(CommunicationManager):
 
         # Internal variables
         self.sample_time = 0.1
+        self.output = None
 
     def print_input(self, msg):
         """The message_parsed callback function that will print ReadInputVoltageCurrentPowerState messages."""
@@ -278,74 +240,135 @@ class ProfileManager(CommunicationManager):
                     cmd(self, row.value, timeout=row.timeout)
             else:
                 cmd_type = getattr(commands, row.command)
-                if not isinstance(row.value, str):
+                if row.value or isinstance(row.value, int):
                     msg = cmd_type(value=row.value)
                 else:
                     msg = cmd_type()
                 self.send_wait(msg, timeout=1)
 
                 if row.timeout:
-                    # Set the load
-                    cmd = LoadSwitch(operation=1)
-                    self.send_wait(cmd, timeout=1)
+                    self.run_mode(timeout=row.timeout, mode=cmd_type)
 
-                    # Run and wait for messages
-                    self.wait_print_input(row.timeout, self.sample_time)
+    def run_mode(self, value=None, timeout=None, mode=None, *args, **kwargs):
+        """Run the given mode (or current mode) for the given timeout while reading the input values.
 
-    @register_internal_command('SampleRate')
-    def set_sample_rate(self, value, **kwargs):
+        If "output" is set save the read inputs to the output file. If no "output" then print the read inputs.
+
+        Steps:
+          * Check the current mode and change the mode if needed.
+          * Turn on the load
+          * Wait the timeout and read input with the set sample_time while saving or printing the output.
+          * Turn off Load
+
+        Args:
+            value (object)[None]: Optional timeout if timeout is not given.
+            timeout (int/float)[None]: Timeout to wait while reading the input.
+            mode (str)[None]: Run in this mode. Mode command (Message, CC, CV, CW, CR) or string "CC", "CV", "CW", "CR".
+        """
+        if timeout is None:
+            timeout = value
+
+        try:
+            mode = mode.MODE_TYPE
+        except (AttributeError, Exception):
+            pass
+
+        # Check the current mode
+        if isinstance(mode, str):
+            current_mode = None
+            msgs = self.send_wait(ReadMode(), msg_type=ReadMode, timeout=1, print_msg=False, print_recv=False)
+            for msg in msgs:
+                if isinstance(msg, ReadMode):
+                    current_mode = msg.mode
+
+            # Change the mode if needed
+            if mode != current_mode:
+                self.send_wait(SetMode(mode=mode), msg_type=CommandStatus, timeout=1, print_msg=False, print_recv=False)
+
+        # Set the load On
+        self.send_wait(LoadSwitch(operation=1), timeout=1)
+
+        # Run and wait for messages
+        if self.output:
+            # If output was set save results to a file
+            msgs = self.wait_save_input(timeout, self.sample_time)
+
+            # Check path
+            if not os.path.exists(os.path.dirname(self.output)):
+                try:
+                    os.makedirs(os.path.dirname(self.output), exist_ok=True)
+                except:
+                    pass
+
+            # Save results
+            with open(self.output, 'a') as f:
+                f.write('Volts,Amps,Watts\n')
+                for msg in msgs:
+                    f.write('{} V, {} A, {} W\n'.format(msg.voltage, msg.current, msg.power))
+        else:
+            # If no output print
+            self.wait_print_input(timeout, self.sample_time)
+
+        # Set the load Off
+        self.send_wait(LoadSwitch(operation=0), timeout=1)
+
+    def get_sample_rate(self):
+        """Return the sampling time delay in hz. The sample rate is (sample_time = 1/value)."""
+        return 1/self.sample_time
+
+    def set_sample_rate(self, value, *args, **kwargs):
+        """Set the sampling time delay in hz. The sample rate is (sample_time = 1/value)."""
         self.sample_time = 1/value
 
-    @register_internal_command('SampleTime')
-    def set_sample_time(self, value, **kwargs):
+    def get_sample_time(self):
+        """Return the sampling time delay."""
+        return self.sample_time
+
+    def set_sample_time(self, value, *args, **kwargs):
+        """Set the sampling time delay."""
         self.sample_time = value
 
-    @register_internal_command('BaudRate')
-    def set_baudrate(self, value, **kwargs):
-        with self.change_connection():
-            self.connection.baudrate = value
+    def get_output(self):
+        """Return the output file."""
+        return self.output
 
-    @register_internal_command('Com')
-    @register_internal_command('Port')
-    def set_port(self, value, **kwargs):
-        with self.change_connection():
-            self.connection.port = value
-
-        if not self.is_connected():
-            try:
-                self.connect()
-                self._enter_connected = True
-            except Exception as err:
-                print('Warning: Could not connect! {}'.format(err), file=sys.stderr)
+    def set_output(self, output, *args, **kwargs):
+        """Set the output file."""
+        self.output = output
 
 
-def main(filename, out=None, com=None, baudrate=None):
+# Custom internal commands
+ProfileManager.register_internal_command('SetupRemote', ProfileManager.setup_remote)
+ProfileManager.register_internal_command('TeardownRemote', ProfileManager.teardown_remote)
+ProfileManager.register_internal_command('Run', ProfileManager.run_mode)
+ProfileManager.register_internal_command('Connect', ProfileManager.connect)
+ProfileManager.register_internal_command('SampleRate', ProfileManager.set_sample_rate)
+ProfileManager.register_internal_command('SampleTime', ProfileManager.set_sample_time)
+ProfileManager.register_internal_command('BaudRate', ProfileManager.set_baudrate)
+ProfileManager.register_internal_command('Com', ProfileManager.set_com)
+ProfileManager.register_internal_command('Port', ProfileManager.set_port)
+ProfileManager.register_internal_command('Output', ProfileManager.set_output)
+
+
+@ProfileManager.register_internal_command('Print')
+def print_status(mngr=None, value='', *args, **kwargs):
+    """Print the Value string."""
+    print(value)
+
+
+def main(filename, com=None, baudrate=None, connection=None, parser=None):
     """Run the given profile."""
-    if out is not None:
-        out = open(out, 'w')
-        sys.stdout = out
-
-    with ProfileManager(com=com, baudrate=baudrate) as mngr:
+    with ProfileManager(com=com, baudrate=baudrate, connection=connection, parser=parser) as mngr:
         mngr.load_profile(filename)
         mngr.run_profile()
 
-    try:
-        sys.stdout = sys.__stdout__
-    except (AttributeError, Exception):
-        pass
-    try:
-        out.close()
-    except (AttributeError, Exception):
-        pass
-
 
 if __name__ == '__main__':
     import argparse
     P = argparse.ArgumentParser('Run a profile.')
     P.add_argument('filename', type=str, help='Profile to read and execute.')
-    P.add_argument('-o', '--out', type=str, default=None, help='Output filename to save to. If None print results.')
     P.add_argument('--com', '-c', type=str, default=None, help='Com port to connect to.')
     P.add_argument('--baudrate', '-b', type=int, default=None, help='Baud rate to connect with.')
     ARGS = P.parse_args()
 
-    main(ARGS.filename, out=ARGS.out, com=ARGS.com, baudrate=ARGS.baudrate)
+    main(ARGS.filename, com=ARGS.com, baudrate=ARGS.baudrate)
