@@ -29,13 +29,16 @@ class CommunicationManager(object):
             parser = self.Parser()
         if connection is None:
             connection = serial.Serial()
-            connection.port = com
-            connection.baudrate = baudrate
+            if com is not None:
+                connection.port = com
+            if baudrate is not None:
+                connection.baudrate = baudrate
             # connection.rts = True  # Documentation states needed. Did not work
             # connection.dtr = True  # Documentation states needed. Did not work
 
         self._parser = None
         self._process = None
+        self._in_enter = False
         self._enter_started = False
         self._enter_connected = False
         self.read_delay = 0.0001
@@ -105,6 +108,35 @@ class CommunicationManager(object):
         if is_connected:
             self.connect()
 
+    def get_baudrate(self):
+        """Return the baudrate."""
+        return self.connection.baudrate
+
+    def set_baudrate(self, value, *args, **kwargs):
+        """Set the baudrate."""
+        with self.change_connection():
+            self.connection.baudrate = value
+
+    def get_com(self):
+        """Return the serial com port."""
+        return self.connection.port
+
+    def set_com(self, value, *args, **kwargs):
+        """Set the serial com port and try to connect."""
+        with self.change_connection():
+            self.connection.port = value
+
+        if not self.is_connected():
+            try:
+                self.connect()
+                if self._in_enter:
+                    self._enter_connected = True
+            except Exception as err:
+                print('Warning: Could not connect! {}'.format(err), file=sys.stderr)
+
+    get_port = get_com
+    set_port = set_com
+
     def is_connected(self):
         """Return if the connection/serial port is connected."""
         try:
@@ -114,7 +146,7 @@ class CommunicationManager(object):
             pass
         return False
 
-    def connect(self, com=None, baudrate=None):
+    def connect(self, com=None, baudrate=None, **kwargs):
         """Connect the connection/serial port."""
         if com is not None or baudrate is not None:
             self.disconnect()
@@ -128,7 +160,7 @@ class CommunicationManager(object):
             if isinstance(self.connection, serial.Serial):
                 self.connection.open()
 
-    def disconnect(self):
+    def disconnect(self, *args, **kwargs):
         """Disconnect the connection/serial port."""
         if isinstance(self.connection, serial.Serial):
             self.connection.close()
@@ -166,10 +198,13 @@ class CommunicationManager(object):
     def read_and_parse(self):
         """Read data from the connection and parse it."""
         try:
-            byts = self.read()
-            if byts:
-                self.parser.parse(byts, self.message_parsed)
-            time.sleep(self.read_delay)
+            if self.is_connected():
+                byts = self.read()
+                if byts:
+                    self.parser.parse(byts, self.message_parsed)
+                time.sleep(self.read_delay)
+            else:
+                time.sleep(0.1)
         except (ConnectionAbortedError, Exception) as err:
             print(str(err), file=sys.stderr)
 
@@ -321,6 +356,7 @@ class CommunicationManager(object):
 
     def __enter__(self):
         """Enter the 'with' context manager."""
+        self._in_enter = True
         if not self.is_connected():
             try:
                 self.connect()
@@ -334,6 +370,7 @@ class CommunicationManager(object):
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         """Exit the 'with' context manager."""
+        self._in_enter = False
         if self._enter_started:
             self._enter_started = False
             self.stop()
